#!/usr/bin/env node
const { build } = require('vite')
const { dirname,resolve } = require('path')
const { Project } = require('ts-morph')

/** @type 'production' | 'development' */
const mode = (process.env.MODE = process.env.MODE || 'production')

const packagesConfigs = [
	'vite.main.config.ts',
	'vite.preload.config.ts',
	'vite.renderer.config.ts',
	'vite.util.config.ts'
]

/**
 * Run `vite build` for config file
 */
const buildByConfig = configFile => build({ configFile, mode })
;(async () => {
	try {
		const totalTimeLabel = 'Total bundling time'
		console.time(totalTimeLabel)

		for (const packageConfigPath of packagesConfigs) {
			const consoleGroupName = `${dirname(packageConfigPath)}/`
			console.group(consoleGroupName)

			const timeLabel = 'Bundling time'
			console.time(timeLabel)

			await buildByConfig(packageConfigPath)

			console.timeEnd(timeLabel)
			console.groupEnd()
			console.log('\n') // Just for pretty print
		}
		console.timeEnd(totalTimeLabel)
	} catch (e) {
		console.error(e)
		process.exit(1)
	}
})()

//
// // 这部分内容具体可以查阅 ts-morph 的文档
// // 这里仅需要知道这是用来处理 ts 文件并生成类型声明文件即可
const project = new Project({

	tsConfigFilePath: resolve(__dirname, './renderer/tsconfig.json'),
})

project.save()