export const MezzFileType = Object.freeze({
	未知: 0,
	图片: 1,
	音频: 2,
	视频: 3,
	word: 4,
	excel: 5,
	ppt: 6,
	pdf: 10,
	eml: 11
})

export const LiveActionType = Object.freeze({
	进入首页: 0,
	进入公司: 1,
	在公司里打开联系人: 2,
	打开联系人: 3,
	联系人留mezz: 4,
	在公司里留mezz: 5,
	完成axon: 6
})

export const MqttStatus = Object.freeze({
	noConnect: 0,
	connected: 1,
	close: 2,
	error: 3
})

interface AtToHtmlConfig {
	cmClass?: string
	contactClass?: string
	accountClass?: string
	ibClass?: string
	obClass?: string
}

interface LiveActionConfig {
	contact?: {
		id: number
		name: string
	}
	account?: {
		id: number
		name: string
	}
	mezz?: string
}

enum AtMark {
	cm = '@',
	contact = '#',
	account = '$'
}

interface matchSortOption {
	same: number
	company_info: Object
	sort: number
	sortype: number

	[name: string]: any
}

interface Perform {
	(cb?: () => void | Promise<any>): void

	timeoutId: number | NodeJS.Timeout

	clear(): void
}

/*
 * 是否是测试模式
 * 一般用于日志打印，测试模式输出日志， 生产模式不输出
 * */
function isDev(): boolean {
	return process.env.NODE_ENV === 'development'
}

/*
 * 去除左右边空(或指定内容)
 * @param {string} str 需要求处理的字符串
 * @param {String} s 可选 需要去除左右边的字符 默认为空白字符；
 * */
function trim(str: string, s?: string): string {

	if (typeof str === 'string') {

		if (!s) return str.trim();

		let l = ltrim(str, s)
		return rtrim(l, s)
	} else {
		return str
	}
}

/*
 * 去除左边空(或指定内容)
 * @param {string} str 需要求处理的字符串
 * @param {String} s 可选 需要去除左边的字符 默认为空白字符；
 * */
function ltrim(str: string, s?: string) {
	if (typeof str === 'string') {
		if (!s) s = '\\s'
		else s = s.replace(/([?.()\[\]*+\\])/g, '\\$1')
		let reg = new RegExp('^(' + s + ')*')
		return str.replace(reg, '')
	} else {
		return str
	}
}

/*
 * 去除右边空(或指定内容)
 * @param {string} str 需要求处理的字符串
 * @param {String} s 可选 需要去除右边的字符 默认为空白字符；
 * */
function rtrim(str: string, s?: string) {
	if (typeof str === 'string') {
		if (!s) s = '\\s'
		else s = s.replace(/([?.()\[\]*+\\])/g, '\\$1')
		let reg = new RegExp('(' + s + ')*$')
		return str.replace(reg, '')
	} else {
		return str
	}
}

const _toString = Object.prototype.toString

/**
 * 是否是普通JavaScript对象。只返回boolean
 */
function isObject(obj: any): boolean {
	return _toString.call(obj) === '[object Object]'
}

//创建at
//{[@cm,11]},{[#contact,124]},{[$account,8451]}
function createAt(mark: AtMark, opt: { name: string; id: number }) {
	return `{[${mark}${opt.name},${opt.id}]}`
}

/**
 * 是否是中文
 * @str
 * @include //包含中文
 * */
function isChn(str: string, include: boolean = false) {
	let re = include ? /[\u4E00-\u9FA5]/g : /^[\u4E00-\u9FA5]+$/
	str = trim(str).replace(/\s+/g, '')
	return re.test(str)
}

/**
 * 是否是英文名字
 * @str
 * */
function isEnn(str: string) {
	let re = /^[A-Za-z][A-Za-z.\s]+[A-Za-z]$/
	str = trim(str).replace(/\s+/g, ' ')
	return re.test(str)
}

function compare(property: string) {
	return function (a: any, b: any) {
		let value1 = a[property]
		let value2 = b[property]
		return value1 - value2
	}
}

/**
 * 替换字符串中的重音字符
 * @param str
 */
function normalizeChars(str: string) {
	let replaceChars: { [key: string]: string } = {
		ъ: '-',
		Ь: '-',
		Ъ: '-',
		ь: '-',
		Ă: 'A',
		Ą: 'A',
		À: 'A',
		Ã: 'A',
		Á: 'A',
		Æ: 'A',
		Â: 'A',
		Å: 'A',
		Ä: 'Ae',
		Þ: 'B',
		Ć: 'C',
		ץ: 'C',
		Ç: 'C',
		È: 'E',
		Ę: 'E',
		É: 'E',
		Ë: 'E',
		Ê: 'E',
		Ğ: 'G',
		İ: 'I',
		Ï: 'I',
		Î: 'I',
		Í: 'I',
		Ì: 'I',
		Ł: 'L',
		Ñ: 'N',
		Ń: 'N',
		Ø: 'O',
		Ó: 'O',
		Ò: 'O',
		Ô: 'O',
		Õ: 'O',
		Ö: 'Oe',
		Ş: 'S',
		Ś: 'S',
		Ș: 'S',
		Š: 'S',
		Ț: 'T',
		Ù: 'U',
		Û: 'U',
		Ú: 'U',
		Ü: 'Ue',
		Ý: 'Y',
		Ź: 'Z',
		Ž: 'Z',
		Ż: 'Z',
		â: 'a',
		ǎ: 'a',
		ą: 'a',
		á: 'a',
		ă: 'a',
		ã: 'a',
		Ǎ: 'a',
		а: 'a',
		А: 'a',
		å: 'a',
		à: 'a',
		א: 'a',
		Ǻ: 'a',
		Ā: 'a',
		ǻ: 'a',
		ā: 'a',
		ä: 'ae',
		æ: 'ae',
		Ǽ: 'ae',
		ǽ: 'ae',
		б: 'b',
		ב: 'b',
		Б: 'b',
		þ: 'b',
		ĉ: 'c',
		Ĉ: 'c',
		Ċ: 'c',
		ć: 'c',
		ç: 'c',
		ц: 'c',
		צ: 'c',
		ċ: 'c',
		Ц: 'c',
		Č: 'c',
		č: 'c',
		Ч: 'ch',
		ч: 'ch',
		ד: 'd',
		ď: 'd',
		Đ: 'd',
		Ď: 'd',
		đ: 'd',
		д: 'd',
		Д: 'D',
		ð: 'd',
		є: 'e',
		ע: 'e',
		е: 'e',
		Е: 'e',
		Ə: 'e',
		ę: 'e',
		ĕ: 'e',
		ē: 'e',
		Ē: 'e',
		Ė: 'e',
		ė: 'e',
		ě: 'e',
		Ě: 'e',
		Є: 'e',
		Ĕ: 'e',
		ê: 'e',
		ə: 'e',
		è: 'e',
		ë: 'e',
		é: 'e',
		ф: 'f',
		ƒ: 'f',
		Ф: 'f',
		ġ: 'g',
		Ģ: 'g',
		Ġ: 'g',
		Ĝ: 'g',
		Г: 'g',
		г: 'g',
		ĝ: 'g',
		ğ: 'g',
		ג: 'g',
		Ґ: 'g',
		ґ: 'g',
		ģ: 'g',
		ח: 'h',
		ħ: 'h',
		Х: 'h',
		Ħ: 'h',
		Ĥ: 'h',
		ĥ: 'h',
		х: 'h',
		ה: 'h',
		î: 'i',
		ï: 'i',
		í: 'i',
		ì: 'i',
		į: 'i',
		ĭ: 'i',
		ı: 'i',
		Ĭ: 'i',
		И: 'i',
		ĩ: 'i',
		ǐ: 'i',
		Ĩ: 'i',
		Ǐ: 'i',
		и: 'i',
		Į: 'i',
		י: 'i',
		Ї: 'i',
		Ī: 'i',
		І: 'i',
		ї: 'i',
		і: 'i',
		ī: 'i',
		ĳ: 'ij',
		Ĳ: 'ij',
		й: 'j',
		Й: 'j',
		Ĵ: 'j',
		ĵ: 'j',
		я: 'ja',
		Я: 'ja',
		Э: 'je',
		э: 'je',
		ё: 'jo',
		Ё: 'jo',
		ю: 'ju',
		Ю: 'ju',
		ĸ: 'k',
		כ: 'k',
		Ķ: 'k',
		К: 'k',
		к: 'k',
		ķ: 'k',
		ך: 'k',
		Ŀ: 'l',
		ŀ: 'l',
		Л: 'l',
		ł: 'l',
		ļ: 'l',
		ĺ: 'l',
		Ĺ: 'l',
		Ļ: 'l',
		л: 'l',
		Ľ: 'l',
		ľ: 'l',
		ל: 'l',
		מ: 'm',
		М: 'm',
		ם: 'm',
		м: 'm',
		ñ: 'n',
		н: 'n',
		Ņ: 'n',
		ן: 'n',
		ŋ: 'n',
		נ: 'n',
		Н: 'n',
		ń: 'n',
		Ŋ: 'n',
		ņ: 'n',
		ŉ: 'n',
		Ň: 'n',
		ň: 'n',
		о: 'o',
		О: 'o',
		ő: 'o',
		õ: 'o',
		ô: 'o',
		Ő: 'o',
		ŏ: 'o',
		Ŏ: 'o',
		Ō: 'o',
		ō: 'o',
		ø: 'o',
		ǿ: 'o',
		ǒ: 'o',
		ò: 'o',
		Ǿ: 'o',
		Ǒ: 'o',
		ơ: 'o',
		ó: 'o',
		Ơ: 'o',
		œ: 'oe',
		Œ: 'oe',
		ö: 'oe',
		פ: 'p',
		ף: 'p',
		п: 'p',
		П: 'p',
		ק: 'q',
		ŕ: 'r',
		ř: 'r',
		Ř: 'r',
		ŗ: 'r',
		Ŗ: 'r',
		ר: 'r',
		Ŕ: 'r',
		Р: 'r',
		р: 'r',
		ș: 's',
		с: 's',
		Ŝ: 's',
		š: 's',
		ś: 's',
		ס: 's',
		ş: 's',
		С: 's',
		ŝ: 's',
		Щ: 'sch',
		щ: 'sch',
		ш: 'sh',
		Ш: 'sh',
		ß: 'ss',
		т: 't',
		ט: 't',
		ŧ: 't',
		ת: 't',
		ť: 't',
		ţ: 't',
		Ţ: 't',
		Т: 't',
		ț: 't',
		Ŧ: 't',
		Ť: 't',
		'™': 'tm',
		ū: 'u',
		у: 'u',
		Ũ: 'u',
		ũ: 'u',
		Ư: 'u',
		ư: 'u',
		Ū: 'u',
		Ǔ: 'u',
		ų: 'u',
		Ų: 'u',
		ŭ: 'u',
		Ŭ: 'u',
		Ů: 'u',
		ů: 'u',
		ű: 'u',
		Ű: 'u',
		Ǖ: 'u',
		ǔ: 'u',
		Ǜ: 'u',
		ù: 'u',
		ú: 'u',
		û: 'u',
		У: 'u',
		ǚ: 'u',
		ǜ: 'u',
		Ǚ: 'u',
		Ǘ: 'u',
		ǖ: 'u',
		ǘ: 'u',
		ü: 'ue',
		в: 'v',
		ו: 'v',
		В: 'v',
		ש: 'w',
		ŵ: 'w',
		Ŵ: 'w',
		ы: 'y',
		ŷ: 'y',
		ý: 'y',
		ÿ: 'y',
		Ÿ: 'y',
		Ŷ: 'y',
		Ы: 'y',
		ž: 'z',
		З: 'z',
		з: 'z',
		ź: 'z',
		ז: 'z',
		ż: 'z',
		ſ: 'z',
		Ж: 'zh',
		ж: 'zh'
	}
	let re = new RegExp(Object.keys(replaceChars).join('|'), 'g')
	return str.replace(re, function (match) {
		return replaceChars[match]
	})
}

/*
 * 把html代码转为Dom元素，使它可以用document.querySelector()或document.querySelectorAll() 来对操作进行选择
 * 该方法用来代替jquery
 * */

export function htmlToDom(html: string, tag: keyof HTMLElementTagNameMap = 'html') {
	let d = document.createElement(tag)
	d.innerHTML = html
	return d
}

class Util {
	trim = trim
	rtrim = rtrim
	ltrim = ltrim
	isDev = isDev
	isObject = isObject
	isChn = isChn
	isEnn = isEnn
	compare = compare
	normalizeChars = normalizeChars
	htmlToDom = htmlToDom

	// 等于 console.log(),不同的是，levlog只在开发模式打印

	devlog(...data: any) {
		if (isDev()) {
			console.warn(...data)
		}
	}

	/**
	 * 创建一个纯函数的缓存版本。
	 */
	cached(fn: Function) {
		let cache = Object.create(null)
		return function (str: string, s2?: string) {
			let key = s2 ? str + s2 : str
			let hit = cache[key]
			return hit || (cache[key] = fn(str, s2))
		}
	}

	//对比两个字符串是否相同,默认不区分大小写
	//@param {String} str1 字符串1
	//@param {String} str2 字符串2
	//@param {Boolean} isMatchCase 开启区分大小写
	//@param {Boolean} isContains 开启包含
	sameText(str1: string, str2: string, isMatchCase: boolean = false, isContains: boolean = false) {
		if (!isMatchCase) {
			//不区分大小写
			str1 = this.toLowerAndTrim(str1)
			str2 = this.toLowerAndTrim(str2)
		}

		if (str1 === str2) {
			return true
		}

		return isContains && (str1.indexOf(str2) > -1 || str2.indexOf(str1) > -1)
	}

	/**
	 * 一维数组去重
	 * @param  {Array} arr  一维数组
	 * @return {any[]} 去重后的一维数组
	 * */
	arrayUnique<T>(arr: T[]): T[] {
		let res = []
		let json: any = {}
		for (let i = 0; i < arr.length; i++) {
			let key: any = arr[i]
			if (typeof arr[i] === 'object' && arr[i] != null) {
				key = JSON.stringify(arr[i])
			}
			if (!json[key] && arr[i]) {
				res.push(arr[i])
				json[key] = true
			}
		}
		return res
	}

	/**
	 * 复制文本，执行该函数，文本将会写入的粘贴板，后续用户可以进行粘贴操作
	 * @param {String} string 需要复制的文本
	 **/
	copyText(string: string): void {
		let oInput: HTMLTextAreaElement | null = <HTMLTextAreaElement>document.getElementById('ici-copy')

		if (!oInput) {
			oInput = <HTMLTextAreaElement>document.createElement('textarea')
			oInput.setAttribute('id', 'ici-copy')
			oInput.setAttribute('type', 'text')
			oInput.style.position = 'fixed'
			oInput.style.bottom = '0'
			oInput.style.left = '-900px'
			oInput.style.opacity = '0'

			oInput.style.zIndex = '-9999'
			document.body.appendChild(oInput)
		}

		oInput.value = string
		oInput.select() // 选择对象
		document.execCommand('Copy') // 执行浏览器复制命令
	}

	/**
	 * 把\r\n换行符转成html的<br>换行符
	 * @param {String} string 字符串
	 * */
	rnTobr(string: string): string {
		if (typeof string === 'string' && string.indexOf('\n') > -1) {
			return string.replace(/\n/g, '<br>')
		} else {
			return string
		}
	}

	/**
	 *
	 * 加粗匹配的关键字
	 * @param str匹配的内容
	 * @param key匹配关键字
	 * @param color 可选 匹配关键字的颜色，默认为绿色;
	 */
	boldKey(str: string, key: string | string[], color?: string): string {
		if (!str) return str
		if (!color) color = 'rgb(77,200,107)'

		if (typeof key === 'string') {
			let s = key.replace(/([()\[\]*\\?.$^])/g, '\\$1').replace(/\+/g, '')
			return str.replace(new RegExp('(' + s + ')', 'ig'), `<b style='color:${color}'>$1</b>`)
		} else if (Array.isArray(key)) {
			let newkeys = key.map(v => {
				return v.replace(/([()\[\]*\\?.$^])/g, '\\$1').replace(/\+/g, '')
			})

			return str.replace(new RegExp('(' + newkeys.join('|') + ')', 'ig'), `<b style='color:${color}'>$1</b>`)
		} else {
			return str
		}
	}

	/**
	 *首字母大字
	 * @param {String}str 字符串
	 * @param {Boolean}isWord 是否以单词为分界
	 */
	capitalize(str: string, isWord?: boolean): string {
		if (typeof str === 'string') {
			str = str.toLowerCase()

			if (isWord) {
				let arr = str.split(/ +/)
				return arr
						.map(n => {
							return n.charAt(0).toUpperCase() + n.slice(1)
						})
						.join(' ')
			} else {
				return str.charAt(0).toUpperCase() + str.slice(1)
			}
		}
		return str
	}

	/**
	 * 判断元素是否在数组里
	 * @param arr 原数组
	 * @param item 元素
	 * @return Boolean
	 */
	inArray(arr: any[], item: any): boolean {
		return arr && arr.length > 0 && item !== undefined && arr.includes(item)
	}

	/**
	 * 从数组中删除元素，此操作会在原数中直接删除，只删除第一个匹配到的元素
	 * @param arr 原数组
	 * @param item 删除项
	 * @return Array 如果删除成功，返回被删除的元素，失败返回undefault
	 */
	removeArrayItem(arr: (string | number)[], item: string | number) {
		if (arr && arr.length) {
			let index = arr.indexOf(item)
			if (index > -1) {
				return arr.splice(index, 1)
			}
		}
	}

	/*
	 * 对象是否为空
	 * @param obj 需要判断的对象
	 * */
	isEmptyObject(obj: Object) {
		//不是对象返回true
		if (isObject(obj)) {
			for (let i in obj) {
				return false
			}
			return true
		}
	}

	/*
	 * 获取域
	 * */
	getDomain(v: string) {
		let str = ''
		if (v) {
			let ret = ''
			if (/^https?:\/\//.test(v)) {
				let arr = v.split('/')
				ret = arr[2]
			} else {
				ret = v
			}
			ret = ret.replace(/[?#/].+/g, '')

			let d = ret.split('.')

			if (d.length > 1) {
				if (d[0] === 'www') {
					d.splice(0, 1)
				}
				str = d.filter(v => !!v).join('.')
			}
		}
		return str
	}

	/*
	 * {[@cm,11]},{[#contact,124]} 格式转换成html,
	 *
	 * */
	atToHtml(str: string, config: AtToHtmlConfig = {}) {
		let defaultConfig: AtToHtmlConfig = {
			cmClass: 'cmAt',
			contactClass: 'contactAt',
			accountClass: 'account-at',
			ibClass: 'ibAt',
			obClass: 'obAt'
		}

		let cof = Object.assign(defaultConfig, config)
		//{[@cm,11]},{[#contact,124]},{[$account,8451]}
		if (typeof str === 'string') {
			if (/{\[(#.+?), ?(\d+), ?(\d+)]}/g.test(str)) {
				str = str.replace(/\{\[(#.+?), ?(\d+), ?(\d+)\]\}/g, '<span class="' + cof.contactClass + '"  data-contact-id="$2" data-company-id="$3">$1</span>')
			} else {
				str = str.replace(/\{\[(#.+?), ?(\d+)\]\}/g, '<span class="' + cof.contactClass + '"  data-contact-id="$2">$1</span>')
			}
			/*   if (/{\[!i, ?([A-Za-z\s]+),?([A-Za-z\s]+),!i]}/g.test(str)) {
							str = str.replace(
								/\{\[(!i), ?([A-Za-z\s]+),?([A-Za-z\s]+),!i\]\}/g,
								'<span class="' + cof.ibClass + '"  data-ib-id="$3">$2</span>'
							)
						}
						if (/{\[!o, ?([A-Za-z\s]+),?([A-Za-z\s]+),!o]}/g.test(str)) {
							str = str.replace(
								/\{\[!o, ?([A-Za-z\s]+),?([A-Za-z\s]+),!o\]\}/g,
								'<span class="' + cof.obClass + '"  data-ob-id="$3">$2</span>'
							)
						}*/
			return str
					.replace(/{\[(@.+?), ?(\d+)\]\}/g, '<span class="' + cof.cmClass + '" data-cm-id="$2">$1</span>')
					.replace(/\{\[\$(.+?), ?(\d+)\]\}/g, '<span class="' + cof.accountClass + '"  data-account-id="$2">$1</span>')
		} else {
			return str
		}
	}

	/*
	 * 实时动作
	 *
	 * */
	liveAction(actionType: ff.LiveActionType, config: LiveActionConfig = {}): string {
		//{[@cm,11]},{[#contact,124]},{[$account,8451]}
		let str: string = ''
		switch (actionType) {
			case LiveActionType.进入首页:
				str = `Just entered the homepage`
				break
			case LiveActionType.进入公司:
				if (config.account) {
					str = `Just entered ${createAt(AtMark.account, config.account)}`
				} else {
					throw {msg: 'config.account 是必须的'}
				}
				break
			case LiveActionType.在公司里打开联系人:
				if (config.contact && config.account) {
					str = `viewed ${createAt(AtMark.contact, config.contact)} information in ${createAt(AtMark.account, config.account)} Account`
				} else {
					throw {msg: 'config.contact 和config.account 是必须的'}
				}
				break
			case LiveActionType.打开联系人:
				if (config.contact) {
					str = `viewed ${createAt(AtMark.contact, config.contact)} information`
				} else {
					throw {msg: 'config.contact 是必须的'}
				}
				break
			case LiveActionType.联系人留mezz:
				if (typeof config.mezz === 'string' && config.contact) {
					str = `left a mazz to ${createAt(AtMark.contact, config.contact)}: ${config.mezz}`
				}
				break
			case LiveActionType.在公司里留mezz:
				if (typeof config.mezz === 'string' && config.account) {
					str = `Leave a mezz with ${createAt(AtMark.account, config.account)}: ${config.mezz}`
				}
				break
			case LiveActionType.完成axon:
				if (config.contact && config.account) {
					str = `completed the axon of ${createAt(AtMark.contact, config.contact)} in company ${createAt(AtMark.account, config.account)}`
				} else {
					throw {msg: 'config.contact 和config.account 是必须的'}
				}
				break
		}

		return str
	}

	// 睡眠操作
	sleep(millisecond: number) {
		return new Promise(resolve => globalThis.setTimeout(resolve, millisecond))
	}

	//超时执行操作,
	// @param time {number} 超时时间，单位毫秒
	// @callback 超过指定的时间后返回cb

	timeoutPerform(time: number): Perform {
		//@ts-ignore
		let self = globalThis.window ? window : global
		let fn = <Perform>function (cb?: () => void) {
			if (fn.timeoutId) {
				//@ts-ignore
				self.clearTimeout(fn.timeoutId)
			}

			fn.timeoutId = self.setTimeout(() => {
				if (typeof cb === 'function') {
					cb()
				}
			}, time)
		}

		fn.timeoutId = 0

		fn.clear = () => {
			if (fn.timeoutId) {
				//@ts-ignore
				self.clearTimeout(fn.timeoutId)
			}
		}

		return fn
	}

	//两次相同操作时，中间添加延时, 完成第一次操作时，在指定时间内不触发相同操作
	// 一般用于防止重复提交
	// @param time {number} 时间间隔，单位毫秒
	//@callback 超过指定的时间后返回cb或promise
	intervalPerform(time: number) {
		let lastTime = 0
		return function (cb: Function) {
			let currentTime = Date.now()
			if (currentTime - lastTime > time) {
				lastTime = currentTime
				if (typeof cb === 'function') cb()
			}
		}
	}

	//拆分全名,分成last_name和first_name
	nameToFirstLastName(name: string, id?: number) {
		let russian_ids = [172, 19, 110, 104, 215, 14, 63, 74, 112, 118, 135, 201, 212, 220, 10, 165, 170]
		let v = name.split(/\s+/),
				len = v.length,
				last = '',
				first = '',
				gender = 1
		if (id && russian_ids.includes(id)) {
			//俄语国家
			let va = v.filter(n => /va$/.test(n))
			if (!va.length) {
				//女
				va = v.filter(n => /v$/.test(n))
				gender = 0
			}
			if (va.length) {
				last = va[va.length - 1]
				first = v.filter(n => n !== last).join(' ')
			}
		}
		if (!last) {
			last = v[len - 1]
			if (isChn(name)) {
				first = name.substring(0, 1)
				last = name.substring(1)
			} else {
				v.forEach((n, i) => {
					if (i < len - 1) first += ' ' + n
				})
			}
		}

		return {last_name: last, first_name: trim(first), gender: gender}
	}

	//匹配关键字排序
	//@param str匹配的内容
	//@param key匹配关键字
	matchSort(data: matchSortOption[], key: string, mkey: string) {
		if (typeof key === 'string') {
			data.forEach((n, i) => {
				if (!n.same) {
					let str1 = trim(n[mkey]),
							arr1 = str1 ? str1.split(' ').filter(d => d) : [],
							arr2 = key.split(' ').filter(d => d),
							arr3 = []
					if (!str1) {
						console.log(i, n)
					}
					arr1.forEach((n, i) => {
						arr2.forEach((a, j) => {
							if (trim(n).toLowerCase() === trim(a).toLowerCase()) {
								arr3.push(n)
							}
						})
					})
					data[i].sort = arr3.length

					if (arr3.length === arr1.length && arr3.length === arr2.length) {
						data[i].same = 1
					} else {
						data[i].same = 0
					}
					data[i].sort = arr3.length + data[i].same
				}
			})

			data.sort(compare('sortype')).reverse()
			data.sort(compare('sort')).reverse()
			//data.sort(compare('same')).reverse()
			return data
		} else {
			return data
		}
	}

	static fileType: { [key: string]: string } = {
		'application/vnd.ms-excel': 'xls', //xls vcs
		'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet': 'xlsx', //xlsx
		'application/pdf': 'pdf', //pdf
		'application/msword': 'doc', //doc
		'application/vnd.openxmlformats-officedocument.wordprocessingml.document': 'docx', //docx
		'application/vnd.ms-powerpoint': 'ppt', //ppt
		'application/vnd.openxmlformats-officedocument.presentationml.presentation': 'pptx' //pptx
	}

	//去除文本中的Emoji表情
	removeEmoji(str: string) {
		return str.replace(
				/[\uD83C|\uD83D|\uD83E][\uDC00-\uDFFF][\u200D|\uFE0F]|[\uD83C|\uD83D|\uD83E][\uDC00-\uDFFF]|[0-9|*|#]\uFE0F\u20E3|[0-9|#]\u20E3|[\u203C-\u3299]\uFE0F\u200D|[\u203C-\u3299]\uFE0F|[\u2122-\u2B55]|\u303D|[\A9|\AE]\u3030|\uA9|\uAE|\u3030/gi,
				''
		)
	}

	/*
	 * 删除特殊符号
	 * */
	delSpecial(s: string): string {
		s = s.replace(/['"\\\/\b\f\n\r\t]/g, '')
		s = s.replace(/[@#$%^&*(){}:"L<>?[]]/g, '')
		return s
	}

	//指定范围随机数
	//@param {Number} min 指定最小值
	//@param {Number} maz 指定最大值
	//@retur {Number} 随机数
	scopeRandom(min: number, max: number): number {
		return Math.floor(Math.random() * (max - min + 1)) + min
	}

	/*
	 * 获取url查询字串参数
	 * @param {string} key 需要获取数据对应的键
	 * @param {string} url 可选，默认为当前页面url
	 * @return {string} 值，值不存在是为空字符串
	 * */
	getUrlParam(key: string, url?: string): string {
		let search = url || document.location.search || document.location.href

		//alert(search);
		let pattern = new RegExp('[?&]' + key + '=([^&]+)', 'g')
		let matcher = pattern.exec(search)
		let items = ''
		if (null != matcher) {
			items = matcher[1]
		}
		return items
	}

	/*
	 * 把对象形式的参数转换成url形式的参数 如{ab:'222',cd:'343'} 转后，ab=222&cd=343
	 * @param params {Object} 参数
	 * @param url {string} 可选
	 * */
	createUrlParam(params: { [name: string]: string | number }, url?: string): string {
		let arr = []

		for (let name in params) {
			arr.push(`${name}=${params[name]}`)
		}
		let str = arr.join('&')
		if (url) {
			str = `${url}?${str}`
		}

		return str
	}

	/**
	 * 是否是字符串
	 */
	isString(v: any): boolean {
		return typeof v === 'string'
	}

	/**
	 * 是否是boolean
	 */
	isBoolean(v: any): boolean {
		return typeof v === 'boolean'
	}

	/*
	 * 验证域名有效性
	 **/
	isDomain(v: string): boolean {
		let reg = /^(?:[^\/=?%&~`@[\]':+!.# ]+\.)+[A-za-z]{2,10}(?:[\/?#][\/=?%&~`@[\]':+!.#\w_ ]*)?$/,
				arr = ['bmp', 'jpg', 'png', 'tif', 'gif', 'pcx', 'tga', 'exif', 'fpx', 'svg', 'psd', 'cdr', 'pcd', 'dxf', 'ufo', 'eps', 'ai', 'raw', 'WMF', 'webp，', 'avif', 'apng', 'JPEG', 'tiff']
		let hz = v.split('.').reverse()[0]
		if (hz && arr.includes(hz)) return false
		return reg.test(v)
	}

	//过滤html
	filterHTMLTag(msg: string): string {
		msg = msg.replace(/<\/?[^>]*>/g, '') //去除HTML Tag
		msg = msg.replace(/[|]*\n/, '') //去除行尾空格
		msg = msg.replace(/&nbsp;/gi, '') //去掉nbsp
		return msg
	}

	/*
	 * 是否是公司内部网，如localhost,127.0.0.1
	 *
	 * */
	isIntranet(URL: string) {
		let intranet = [
			'ff://',
			'devtools://',
			'http://192.168.2.23',
			'https://192.168.2.23',
			'http://localhost',
			'http://127.0.0.1',
			'file:///',
			'https://test3.hydrophis.cn',
			'https://fms3.hydrophis.cn',
			'http://bbs.hydrophis.cn'
		]

		return intranet.some(v => URL.indexOf(v) === 0)
	}

	//获取完整的姓名
	getFullName(firstName: string, lastName: string): string {
		return /[A-Za-z]+/.test(lastName) ? firstName + ' ' + lastName : lastName + '' + firstName
	}

	/*
	 * 微信联系方式类型，有三种 id/二维码/图片
	 * 通过传入字符串判断微信是什么类型
	 * returnn qr:二维码；img:图片，id微信id/公众号名
	 * */
	getWeChatType(str: string): 'qr' | 'img' | 'id' {
		if (str.includes('u.wechat.com/') || str.includes('weixin.qq.com/r/') || str.includes('work.weixin.qq.com/ct/')) {
			return 'qr'
		} else if (str.includes('http://open.weixin.qq.com/qr/code?username=')) {
			return 'img'
		} else {
			return 'id'
		}
	}

	/*
	 * 生成唯一id
	 * */
	private static lastKey = Date.now()

	getUniqueKey() {
		Util.lastKey++
		return Util.lastKey.toString()
	}

	/*
	 * 转为小写，并去除左右空白字符
	 * */
	toLowerAndTrim(v: any) {
		if (typeof v === 'string') {
			let str = String.prototype.toLowerCase.call(v)
			return trim(str)
		} else if (typeof v === 'number') {
			return Object.prototype.toString.call(v)
		} else {
			console.warn('toLowerAndTrim函数只接收string和number类型', v)
			return ''
		}
	}

	/*
	 * 异步延迟,有两种参数
	 * @param {number} time 延迟的时间，单位毫秒
	 * @param {number} maxTime 随机最大值，当有该值时，参数time为随机最小值，有该参数时，会调用scopeRandom函数
	 * */
	asyncDelay(time: number, maxTime?: number) {
		if (maxTime) {
			time = this.scopeRandom(time, maxTime)
		}
		return new Promise(resolve => {
			globalThis.setTimeout(() => {
				resolve(void 0)
			}, time)
		})
	}

	// 将字符串单词首字母转为大写
	titleCase(str: string) {
		let newArr = str.toLowerCase().split(' ')
		for (let i = 0; i < newArr.length; i++) {
			if (!newArr[i]) {
				continue
			}
			newArr[i] = newArr[i][0].toUpperCase() + newArr[i].substring(1, newArr[i].length)
		}
		return newArr.join(' ')
	}

	//是否是支持的桌面文件
	isDesktopFiles(type: string) {
		if (Util.fileType[type] || type.indexOf('image/') === 0) {
			return true
		} else {
			return false
		}
	}

	//是否是execl文件
	isExecl(type: string) {
		return ['application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'application/vnd.ms-excel'].includes(type)
	}

	formatCapital(num: string) {
		if (!/^(\d+,*)+$/.test(num)) return num
		if (!num) return 0
		let cap = Number(num.replace(/,/g, ''))
		let obj = {str: String(cap), dw: ''},
				len = obj.str.length
		if (len < 3) return obj.str
		else if (len > 3 && len <= 6) obj = {str: (cap / 1000).toFixed(2), dw: 'K'}
		else if (len > 6 && len <= 9) obj = {str: (cap / 1000000).toFixed(2), dw: 'M'}
		else if (len > 9) obj = {str: (cap / 1000000000).toFixed(2), dw: 'B'}
		let arr = obj.str.split('.')
		if (!parseInt(arr[1])) obj.str = arr[0]
		return obj.str + obj.dw
	}

	/**
	 * 获取元素距离顶部的高度
	 * @param el
	 */
	getElementOffset(el: HTMLElement) {
		let actualTop = el.offsetTop
		let actualLeft = el.offsetLeft
		let current = <HTMLElement>el.offsetParent
		while (current !== null) {
			actualTop += current.offsetTop
			actualLeft += current.offsetLeft
			current = <HTMLElement>current.offsetParent
		}
		return {
			top: actualTop,
			left: actualLeft
		}
	}

	//获取网址
	getWebSite(text: string) {

		let items = {
			data: [] as string[],
			text: ''
		}

		let r: RegExpExecArray | null = null
		let strs: string[] = []
		let n = 0
		do {
			n++
			r = /http(s)?:\/\/[^.]+\.[^\s 　]+/i.exec(text)
			if (r && r[0]) {
				text = text.replace(r[0], '')
				strs.push(r[0])
			}
		} while (r && n < 50)

		items.text = text

		if (strs.length) {
			items.data.push(...strs)
		}

		let reg: RegExp =
				/[^\s 　]+(?:\.[^\s 　]+){0,2}\.(?:partners|tech|com|arpa|edu|gov|int|mil|net|org|biz|info|pro|name|museum|coop|aero|xxx|idv|mobi|asia|jobs|tel|travel|cloud|network|digital|me|ax|bl|bq|cat|cw|gb|al|mf|rs|su|sx|dz|af|ar|ae|aw|om|az|eg|et|ie|ee|ad|ao|ai|ag|at|au|mo|bb|pg|bs|pk|py|ps|bh|pa|br|by|bm|bg|mp|bj|be|is|pr|ba|pl|bo|bz|bw|bt|bf|bi|bv|kp|gq|dk|de|tl|tp|tg|dm|do|ru|ec|er|fr|fo|pf|gf|tf|va|ph|fj|fi|cv|fk|gm|cg|cd|co|cr|gg|gd|gl|ge|cu|gp|gu|gy|kz|ht|kr|nl|an|hm|hn|ki|dj|kg|gn|gw|ca|gh|ga|kh|cz|zw|cm|qa|ky|km|ci|kw|cc|hr|ke|ck|lv|ls|la|lb|lt|lr|ly|li|re|lu|rw|ro|mg|im|mv|mt|mw|my|ml|mk|mh|mq|yt|mu|mr|us|um|as|vi|mn|ms|bd|pe|fm|mm|md|ma|mc|mz|mx|nr|np|ni|ne|ng|nu|no|nf|na|za|aq|gs|eu|pw|pn|pt|jp|se|ch|sv|ws|yu|sl|sn|cy|sc|sa|cx|st|sh|kn|lc|sm|pm|vc|lk|sk|si|sj|sz|sd|sr|sb|so|tj|tw|th|tz|to|tc|tt|tn|tv|tr|tm|tk|wf|vu|gt|ve|bn|ug|ua|uy|uz|es|eh|gr|hk|sg|nc|nz|hu|sy|jm|am|ac|ye|iq|ir|il|it|in|id|uk|vg|io|jo|vn|zm|je|td|gi|cl|cf|cn|yr)(?:[/?&#][^\n\f\r\t\v]*|\b)/m
		let res: RegExpExecArray | null = null
		let num = 0
		do {
			num++
			res = reg.exec(items.text)
			if (res && res[0]) {
				items.data.push('https://' + res[0])
				items.text = items.text.replace(res[0], '')
			}
		} while (res && num < 50)

		return items
	}

	//字符串转正则
	strToRegExp(str: string) {
		str = ff.util.trim(str)
		//判断是不是正则
		if (str.indexOf('/') === 0 && str.lastIndexOf('/') === str.length - 1) {
			return new RegExp(str.substring(1, str.length - 1))
		} else if (str.indexOf('/') === 0 && str.lastIndexOf('/i') === str.length - 2) {
			return new RegExp(str.substring(1, str.length - 2), str.substring(str.length - 1))
		}
	}

}

let funs = new Util()

export {Util}

export default funs
