import {connect, MqttClient} from 'mqtt'

//@ts-ignore
import * as CryptoJS  from 'crypto-js'

import {ipcMain, webContents} from 'electron'


import {IClientOptions} from 'mqtt/types/lib/client'

//主题列表
//topic 主题名称
//number[]; 是webContentId数组
interface Topics {
  [topic: string]: number[]
}

export interface MqttMiddlewareOption {
  regExp: RegExp
  method(data: any, topic: string, regExp: RegExp): any
}

export interface Mqttoptions{

  route:string|number, //路由，线路，同线路才可以通信
  project:string|number, //项目，正常mqtt需要同项目可以接收，除前缀是'@/'的主题订阅可以跨项目
  userId:number,//用户id,可以排除自己发送的id

  middlewares?:MqttMiddlewareOption[]
}
/*
 * mqtt main端
 * @param {String} serverUrl mqtt服务地址
 * @param {Array} 中间件
 * */
class MqttMain {
  client: MqttClient | null = null //实例化的对象
  isClose: boolean = false //是否手动关闭
  status: ff.MqttStatus = ff.MqttStatus.noConnect
  accesskey: string = 'LTAIOtrvqkIYauAq'
  secretKey: string = '6J5b2UZt48Jstc8lpSEgPXhvpWamCH'
  instanceId: string = 'post-cn-zz11pk17f01' //mqtt实例id
  serverUrl: string = 'tls://post-cn-zz11pk17f01.mqtt.aliyuncs.com:8883' //mqtt服务器地址

  label: string = 'fms'
  route:string|number = ''
  project:string|number = ''
  projectTopicPrefix: string= ''
  commonPrefix: string = '@/' //通用前缀符，出现通用前缀符时，不会加项目id;
  userId: number = 0

  middleware: MqttMiddlewareOption[] =[] //中间件

  topics: Topics = {}

  constructor() {

    this.client = null
    this.isClose = false

    //删除客户端
    ipcMain.on('remove-mqtt-client', (e: any, webContentId: number) => {
      console.log('删除对应的客户端', webContentId)

      let unSubTopic: string[] = []
      for (let i in this.topics) {
        let index = this.topics[i].findIndex(id => webContentId === id)
        if (index > -1) {
          this.topics[i].splice(index, 1)

          if (this.topics[i].length === 0) {
            //当该主题已经没有id,直接取消订阅
            delete this.topics[i]
            unSubTopic.push(this.projectTopicPrefix + '/' + i)
          }
        }

        if (unSubTopic.length && this.client) {
          this.client.unsubscribe(unSubTopic)
        }
      }
    })

    /*
     * 发布一个本地的mqtt,只会通知本机的订阅者
     * @param {String} topic 主题
     * @param {mixni} message 内容
     * */
    ipcMain.on('mqtt-local-publich', (e: any, topic: string, message: any) => {
      this.localPublish(topic, message)
    })

    //主题订阅
    ipcMain.on('mqtt-subscribes', (e: any, tops: string[], webContentId: number) => {
      this.subscribes(tops, webContentId)
    })

    //取消主题订阅
    ipcMain.on('mqtt-unsubscribes', (e: any, tops: string[], webContentId: number) => {
      this.unsubscribes(tops, webContentId)
    })

    //发布一个线上主题
    ipcMain.on('mqtt-publich', (e: any, topic: string, messaage: any) => {
      this.publish(topic, messaage)
    })

    //渲染层获取状态触发
    ipcMain.on('$mqtt/status', () => {
      this.statusChange()
    })
  }

  //连接mqtt服务器
  connect(options:Mqttoptions) {
    if(options.middlewares){
      this.middleware = options.middlewares
    }

    this.route = options.route
    this.project = options.project
    this.userId = options.userId

    this.projectTopicPrefix = `${this.label}/${this.route}/${this.project}`

    this.isClose = false //取消主动关闭

    let clientId = `GID-WEB@@@${this.userId}|${this.projectTopicPrefix}|${Date.now()}`

    let username = `Signature|${this.accesskey}|${this.instanceId}`
    let password = CryptoJS.HmacSHA1(clientId, this.secretKey).toString(CryptoJS.enc.Base64)
    console.log(clientId, username, password)
    let opt: IClientOptions = {
      reconnectPeriod: 3000, //重连时间
      keepalive: 90,
      clientId: clientId,
      username: username,
      password: password,
      clean: true,
      resubscribe: false
    }

    if (this.client) this.client.end(true)

    this.client = connect(this.serverUrl, opt)

    this.client.on('connect', () => {
      this.status = ff.MqttStatus.connected
      this.statusChange()

      this.client?.subscribe(this.label)

      let tops = Object.keys(this.topics).map(str => this.projectTopicPrefix + '/' + str)
      if (tops.length) {
        this.client?.subscribe(tops)
      }
    })

    this.client.on('reconnect', (e: any) => {
      ff.util.devlog('重新连接')
    })

    this.client.on('offline', (e: any) => {
      ff.util.devlog('客户端脱机时发出 offline', e)
      this.status = ff.MqttStatus.error
      this.statusChange()
    })

    this.client.on('close', () => {
      ff.util.devlog('断开连接后发出 close')
      if (this.isClose) {
        //只监听手动关闭的
        this.status = ff.MqttStatus.close
        this.statusChange()
      }
    })

    this.client.on('error', (e: any) => {
      this.status = ff.MqttStatus.error
      this.statusChange()

      ff.util.devlog('即时通讯连接失败', e.toString())
    })

    this.client.on('message', (topic: string, message: any) => {
      topic = ff.util.rtrim(topic, '/')
      if (topic === this.projectTopicPrefix) return

      topic = topic.replace(this.projectTopicPrefix + '/', '')
      //二进制buffer转换
      //本地不使用二进制buffer转换
      let msg
      try {
        // message is Buffer
        msg = JSON.parse(message.toString())
      } catch (e) {
        msg = message.toString()
      }

      this.sortingMessages(topic, msg)
    })
  }



  /*
   * 发布一个本地的mqtt,只会通知本机的订阅者
   * @param {String} topic 主题
   * @param {mixni} message 内容
   * */
  localPublish(topic: string, message?: any) {
    this.sortingMessages(topic, message, true)
  }

  //当mqtt状态发生改变时
  statusChange() {
    ff.gc.send('系统mqtt连接状态变化', this.status)
  }

  /*
   * 分拣消息,分配消息通知到指定的位置
   * @param {String} topic 主题
   * @param {mixin} message 消息
   * @param {Boolean} isLocal 是否是本地通知,本地通知会跳过中间件和二进制转换
   * */
  async sortingMessages(topic: string, msg: any, isLocal: boolean = false) {
    if (!isLocal && msg && msg.$user_id && this.userId && msg.$user_id === this.userId) {
      console.log(
        '%c在线mqtt(操作员是自己，不真实通知): ' + topic,
        'color:yellow;background-color:#319537; padding: 1px 2px; border-radius: 3px;',
        msg
      )
      return
    }

    //获取订阅主题的数量
    let clientIds: number[] = []

    //直接配置的，没有通配符
    if (this.topics[topic] && this.topics[topic].length) {
      clientIds = clientIds.concat(this.topics[topic])
    }
    //通配符主题
    for (let i in this.topics) {
      if (i.indexOf('/+') !== -1) {
        let d = i.replace(/\+/g, '[^/]+').replace(/\$/g, '\\$')
        let reg = new RegExp('^' + d + '$')

        if (reg.test(topic)) {
          if (this.topics[i] && this.topics[i].length) {
            clientIds = clientIds.concat(this.topics[i])
          }
        }
      }
    }

    if (!clientIds.length && !this.topics[topic]) {
      ff.util.devlog('没有订阅该主题', topic)
      return
    }

    ff.util.devlog((isLocal ? '本地' : '在线') + 'mqtt通知', topic, msg instanceof Object ? '' : msg)

    //本地不使用中间件
    if (!isLocal) {
      let index = this.middleware.findIndex(ware => ware.regExp.test(topic))

      if (index !== -1) {
        //有中间件
        try {
          msg = await this.middleware[index].method(msg, topic, this.middleware[index].regExp)

          if (msg === false) {
            ff.util.devlog('经过mqtt中间件,不返回值', topic, msg)
            return
          }

          ff.util.devlog('经过mqtt中间件后的值', topic, msg)
        } catch (e) {
          throw e
        }
      }
    }

    clientIds = ff.util.arrayUnique(clientIds)

    clientIds.forEach(id => {
      let webc = webContents.fromId(id)
      webc && webc.send('mqtt-message', topic, msg, isLocal)
    })
  }

  //关闭通讯
  close() {
    this.isClose = true
    this.client &&
      this.client.end(undefined, undefined, () => {
        this.topics = {}
      })
  }

  //订阅
  subscribes(topics: string[], webContentId: number) {
    let subTopics: string[] = []

    topics.forEach(topic => {
      if (this.topics[topic]) {
        !this.topics[topic].includes(webContentId) && this.topics[topic].push(webContentId)
      } else {
        this.topics[topic] = [webContentId]

        if (topic.indexOf(this.commonPrefix) === 0) {
          subTopics.push(`${this.label}/${this.route}/${topic}`)
        } else {
          subTopics.push(this.projectTopicPrefix + '/' + topic)
        }
      }
    })

    if (subTopics.length && this.client) {
      this.client.subscribe(subTopics, {qos: 0}, e => {
        if (e) {
          console.log('订阅失败', e)
        }
      })
    }
  }

  //取消订阅
  unsubscribes(topics: string[], webContentId: number) {
    let unsubTopic: string[] = []

    topics.forEach(topic => {
      if (!this.topics[topic]) return
      let index = this.topics[topic].findIndex(id => id === webContentId)
      if (index > -1) {
        this.topics[topic].splice(index, 1)
      }
      if (this.topics[topic].length === 0) {
        delete this.topics[topic]

        //排除联系人订阅,因为联系人已经使用了公共订阅 contact/update
        if (/contact\/\d+\/update/.test(topic)) return

        if (topic.indexOf(this.commonPrefix) === 0) {
          unsubTopic.push(`${this.label}/${this.route}/${topic}`)
        } else {
          unsubTopic.push(this.projectTopicPrefix + '/' + topic)
        }
      }
    })

    if (unsubTopic.length) {
      this.client && this.client.unsubscribe(unsubTopic)
    }
  }

  publish(topic: string, message: any) {
    this.client && this.client.publish(this.projectTopicPrefix + '/' + topic, message, {qos: 0})
  }
}

export default MqttMain
