/// <reference path="../types.d.ts"/>
import gc from './src/gc'
import mqtt from './src/mqtt'
import util, {MezzFileType, MqttStatus, LiveActionType, Util} from '../common/util'
import Ajax from '../common/ajax'
import crawlerUtil, {CrawlerUtil} from './src/crawler/utils'
import CrawlerDb from './src/crawler/database'
import axios, {AxiosRequestConfig, AxiosStatic, Canceler as C} from 'axios'
import NameSplit from './src/name-split'
import error from '../common/error'
import DateTime from './src/date-time'
import ContactInfo from './src/contactInfo'
import Linkedin, { LinkedinAccount} from './src/linkedin'
import CountryClass, {Country as Co, CountryBase as CB} from './src/country'
import * as Gc from "../gc.type";



/**
 * @desc 渲染进程全局变量
 * @author 黄敏
 * @创建时间 2022-08-02 15:15
 **/

type ValueOf<T> = T[keyof T]

declare global {
  namespace Axios {
    type Canceler = C
    type RequestConfig = AxiosRequestConfig
  }

  namespace ff {
    type MezzFileType = ValueOf<typeof MezzFileType>
    type MqttStatus = ValueOf<typeof MqttStatus>
    type LiveActionType = ValueOf<typeof LiveActionType>

    //mqtt全局
    namespace mqtt {
      interface Subs {
        [topic: string]: (...data: any) => void
      }
    }

    //gc全局
    namespace gc {
      type KeyboardShortcut = Gc.KeyboardShortcut
      type Client = Gc.Client
      type IpcTopic = Gc.IpcTopic
      type Subscribes = Gc.Subscribes
    }

    namespace country {
      type CountryBase = CB
      type Country = Co
    }
    namespace linkedin {
      type AccountStatus = LinkedinAccount
    }
  }
}

declare global {
  var extensions:Crawler.Extension[]
  var ff: Readonly<{
    MezzFileType: typeof MezzFileType //常量，mezz文件类型
    MqttStatus: typeof MqttStatus
    LiveActionType: typeof LiveActionType
    Ajax: typeof Ajax
    util: Util
    error: typeof error
    gc: typeof gc
    mqtt: typeof mqtt
    axios: AxiosStatic
    nameSplit: NameSplit
    dateTime: typeof DateTime
    contactInfo: typeof ContactInfo
    linkedin: Linkedin
    country: CountryClass
    crawler: {
      util: CrawlerUtil
      db: CrawlerDb
    }
  }>
}


window.ff = Object.freeze({
  MezzFileType,
  MqttStatus,
  LiveActionType,
  error,
  util,
  Ajax,
  gc,
  mqtt,
  axios,
  country: new CountryClass(),
  linkedin: new Linkedin,
  nameSplit: new NameSplit(),
  dateTime: DateTime,
  contactInfo: ContactInfo,
  crawler: {
    util: crawlerUtil,
    db: new CrawlerDb()
  }
})
