import contactConfig,{ConfigArr} from './contactInfo/contact-info'

/**
 * @用途
 * @author 黄敏
 * @创建时间 2022-08-10 15:39
 **/


interface ItemReturn {
	text: string //匹配完后剩余的文本
	data: string[] //匹配的结果
}

class ContactInfo {

	static config = contactConfig
	static configArr = ConfigArr

	static validate(type: f2.info.Type, info: string) {
		let conf = ContactInfo.config[type]
		if (conf) {
			if (conf.regular) return conf.regular.test(info)
			else return true
		} else {
			return false
		}
	}

	/**
	 * 新建一个新的联系方式
	 * @param type 联系方式类型
	 * @param info 联系试式内容
	 * @param countryId 国家id
	 * */
	static newInfo(type: f2.info.Type, info: string = '', countryId: number = 0): f2.info.Info {
		return {
			type: type,
			info: info,
			country_id: countryId,
			remark: '',
			ext: '',
			phone_type: 0
		}
	}

	//获取 Email
	static getEmail(text: string): ItemReturn {
		let emailReg = /[\w-.]+?(@|\(at\))[\w-]+?\.[\w-]+(\.[\w-]+)?/gi
		let res: any
		let strs: string[] = []
		let num = 0
		do {
			num++
			res = emailReg.exec(text)

			if (res) {
				strs.push(res[0])
			}
		} while (res && num < 50)
		return {
			text: text,
			data: strs
		}
	}


	//获取 Wechat
	static getWechat(text: string): ItemReturn {
		let wechat = /(wechat|微信号?是?)[: 　：.]?[: 　：.]?(\w{4,20})/i
		let reg1 = /https?:\/\/open\.weixin\.qq\.com\/qr\/code\?username=[\w-+]+/
		let reg2 = /https?:\/\/u\.wechat\.com\/[\w-+]+|https?:\/\/weixin\.qq\.com\/r\/[\w-+]+/

		let res: any
		let strs: string[] = []
		let num = 0
		do {
			res = reg1.exec(text)
			num++
			if (res) {
				strs.push(res[0])
				text = text.replace(res[0], '')
			}
		} while (res && num < 50)
		num = 0
		do {
			res = reg2.exec(text)
			num++
			if (res) {
				strs.push(res[0])
				text = text.replace(res[0], '')
			}
		} while (res && num < 50)
		num = 0
		do {
			num++
			res = wechat.exec(text)
			if (res) {
				strs.push(res[2])
				text = text.replace(res[2], '')
			}
		} while (res && num < 50)
		// if (strs.length > 2) {
		// 	strs.splice(2,strs.length)
		// }

		return {
			text: text,
			data: strs
		}
	}

	//获取 qq
	static getQq(text: string): ItemReturn {
		//qq1
		let qq1 = /qq[: 　：.]?[: 　：.]??(\d{5,14})($|\D)/i
		//qq2 从qq邮箱拿到qq
		let qq2 = /(\d{6,14})@qq.com/g

		let res: any
		let strs: string[] = []
		let num = 0
		do {
			res = qq1.exec(text)
			num++
			if (res) {
				strs.push(res[1])
				text = text.replace(res[1], '')
			}
		} while (res && num < 50)
		num = 0
		do {
			num++
			res = qq2.exec(text)
			if (res) {
				strs.push(res[1])
				text = text.replace(res[1], '')
			}
		} while (res && num < 50)

		return {
			text: text,
			data: strs
		}
	}

	//获取 getSkype
	static getSkype(text: string): ItemReturn {
		//qq1
		let skype = /skype[: 　：.]?[: 　：.]?(\w[\w.]{1,18}\w)/i

		let res: any
		let strs: string[] = []
		let num = 0 //循环不能超过50
		do {
			num++
			res = skype.exec(text)
			if (res) {
				strs.push(res[1])
				text = text.replace(res[1], '')
			}
		} while (res && num < 50)

		return {
			text: text,
			data: strs
		}
	}

	//获取 Phone
	static getPhone(text: string): ItemReturn {
		let strs: string[] = []

		//是否是日期
		let isDate = /\D?((1[89]\d\d|2[01]\d\d)[- ._/]?(0[1-9]|1[012])[- ._/]?(0[1-9]|[12][0-9]|3[01])|((0[1-9]|[12][0-9]|3[01]))[- ._/]?(0[1-9]|[12][0-9]|3[01])[- ._/]?(1[89]\d\d|2[01]\d\d))/

		//场景1，单独匹配该格式 +213 (029) 27 22 80 / 27 22 06 或 (029) 27 22 80 / 27 22 06
		//把国家区号和城市区号拉过去    国家区号   区号     电话1        电话2
		//["(029) 27 22 80/27 22 06", undefined, "029", "27 22 80", "27 22 06", index: 0, input: "(029) 27 22 80/27 22 06awefwaefwaef", groups: undefined]
		let scenario1 = /(?:(\+\d+) {1,2})?[（(](\d+)[)）] ((?:\d+ ){1,3}\d+) {0,3}\/ {0,3}((?:\d+ ){1,3}\d+)/
		let scenario1Global = new RegExp(scenario1, 'g')
		let scenario1Phone = text.match(scenario1Global)
		if (scenario1Phone) {
			scenario1Phone.forEach(v => {
				text = text.replace(v, '')
				let data = v.match(scenario1)
				if (data) {
					let phone1 = `${data[1] ? data[1] + ' ' : ''}${data[2]}${data[3]}`
					let phone2 = `${data[1] ? data[1] + ' ' : ''}${data[2]}${data[4]}`
					strs.push(phone1, phone2)
				}
			})
		}
		// 匹配没有空格的电话号码
		//+94703030666 0702020666
		let scenario2 = /(\+\d{1,4} {0,3}\d{6,13}) +(\d{6,13})/
		let scenario2Global = new RegExp(scenario2, 'g')
		let scenario2Phone = text.match(scenario2Global)
		if (scenario2Phone) {
			scenario2Phone.forEach(v => {
				text = text.replace(v, '')

				let data = v.match(scenario2)
				if (data) {
					let phone1 = data[1]
					let areacode = ff.country.getPhoneAreaCode(data[1])
					let phone2 = (areacode ? `+${areacode}` : '') + data[2]

					strs.push(phone1, phone2)
				}
			})
		}

		//匹配 +7(495)5329101 / 7(495)5329101
		let scenario3 = /(\+?\d{1,4} {0,3}\(\d{2,5}\)\d{6,13})/
		let scenario3Global = new RegExp(scenario3, 'g')
		let scenario3Phone = text.match(scenario3Global)
		if (scenario3Phone) {
			scenario3Phone.forEach(v => {
				text = text.replace(v, '')

				let data = v.match(scenario3)
				if (data) {
					strs.push(data[1])
				}
			})
		}

		//匹配带横杠的格式
		//970-562-1824 86-970-562-1824 +86-970-5162-1824 +86 970-5262-1824 (86) 970-5362-1824 (86)-970-5462-1824
		//(+86) 970-5562-1824 +(86) 970-5662-1824 +(86).970-5762-1824 86.970-5862-1824 +86.970-5962-1824 (86).970-6062-1824
		let hengGang = /(\+?\d+[ \-.]{0,2}|\(\+?\d+\)[- .]{0,2}|\+\(\d+\)[- .]{0,2})?(\d+-){2,3}\d+/g
		let hengGangPhone = text.match(hengGang)
		if (hengGangPhone) {
			hengGangPhone.forEach(v => {
				text = text.replace(v, '')
				strs.push(v)
			})
		}

		//通用匹配
		let common = /\+?(?:\d+[. -])?(?:[（(]\d{1,5}[)）] *)?\d[\d\- .]{4,16}\d+/g
		let commonPhone = text.match(common)
		if (commonPhone) {
			commonPhone.forEach(v => {
				text = text.replace(v, '')
				strs.push(v)
			})
		}

		if (strs.length) {
			//过滤日期的格式
			strs = strs.filter(v => {
				let arr = v.split(/ +/)
				let noSp = v.replace(/[. ,+]/, '')
				//不能少于7位
				if (noSp.length < 7) {
					return false
				}

				//美国的电话不能小于10位
				if (arr[0] === '1' && arr[1] && noSp.length < 10) {
					return false
				}

				if (isDate.test(v)) {
					return false
				} else if (v.replace(/\D/g, '').length > 14) {
					return false
				} else {
					return true
				}
			})
		}

		//如果国家区有括号号（86）转成+86
		if (strs.length) {
			strs = strs.map(v => {
				return v.replace(/^\+?\(\+?(\d+)\)/, '+$1')
			})
		}

		//去重操作
		let newStrs: string[] = []
		strs.forEach(v => {
			let d = v.replace(/\D/g, '')

			let isReplace = false

			let index = newStrs.findIndex(v2 => {
				let d2 = v2.replace(/\D/g, '')
				if (d === d2) {
					if ((v.indexOf('+') === 0 && !v2.includes('+')) || v.length > v2.length) {
						isReplace = true
					}
					return true
				} else if (d2.includes(d)) {
					return true
				} else if (d.includes(d2)) {
					isReplace = true
					return true
				}

				return false
			})

			if (index === -1) {
				newStrs.push(v)
			} else if (isReplace) {
				newStrs.splice(index, 1, v)
			}
		})

		return {
			text: text,
			data: newStrs
		}
	}


	/*
	 * 把 ItemReturn.data转成contact Info
	 * */
	static getDataInfo(arr: string[], type: f2.info.Type) {
		arr = ff.util.arrayUnique(arr)
		return arr.map(f => {
			try {
				f = decodeURI(f)
			} catch (e) {
				console.log('decodeURI失败')
			}
			return {type: type, info: f}
		})
	}

	/**
	* 在url中提取联系方式
	* */
	static textToInfos(text: string) {

		let infos: f2.info.Info[] = []

		let d = ff.contactInfo.getEmail(text)
		infos.push(...ContactInfo.getDataInfo(d.data, 'email'))

		d = ff.contactInfo.getWechat(d.text)
		infos.push(...ContactInfo.getDataInfo(d.data, 'wechat'))

		d = ff.contactInfo.getQq(d.text)
		infos.push(...ContactInfo.getDataInfo(d.data, 'qq'))

		d = ff.contactInfo.getSkype(d.text)
		infos.push(...ContactInfo.getDataInfo(d.data, 'skype'))

		d = ff.contactInfo.getPhone(d.text)
		infos.push(...ContactInfo.getDataInfo(d.data, 'phone'))

		return {
			infos,
			text:d.text
		}
	}
}

export default ContactInfo
