/**
 * Created by huangmin on 2017/4/26 0026.
 *
 * 时间类,
 *
 */

type DataParam = number | string | Date

const monthEn: { [key in f2.Month]: { name: string; abbr: string; month: f2.Month | 0 } } = {
	1: {month: 1, name: 'January', abbr: 'Jan.'},
	2: {month: 2, name: 'February', abbr: 'Feb.'},
	3: {month: 3, name: 'March', abbr: 'Mar.'},
	4: {month: 4, name: 'April', abbr: 'Apr.'},
	5: {month: 5, name: 'May', abbr: 'May'},
	6: {month: 6, name: 'June', abbr: 'Jun.'},
	7: {month: 7, name: 'July', abbr: 'Jul.'},
	8: {month: 8, name: 'August', abbr: 'Aug.'},
	9: {month: 9, name: 'September', abbr: 'Sep.'},
	10: {month: 10, name: 'October', abbr: 'Oct.'},
	11: {month: 11, name: 'November', abbr: 'Nov.'},
	12: {month: 12, name: 'December', abbr: 'Dec.'}
}

class DateTime {
	static monthOptions = Object.values(monthEn)

	static createDate(d: DataParam) {
		const type = typeof d
		let date
		if (type === 'number') {
			date = new Date(d)
		} else if (type === 'string') {
			date = new Date(d)
		} else if (type === 'object' && d instanceof Date) {
			date = d
		} else {
			date = new Date()
		}

		if (date.toString() === 'Invalid Date') {
			throw 'Invalid Date'
		}
		return date
	}

	//获取时间 传入格式，h:i:s
	static getHm(d: DataParam, format = 'h:i'): string {
		let date = DateTime.createDate(d)

		format = format.toLowerCase()
		let hours = DateTime.zeroFill(date.getHours())
		let minutes = DateTime.zeroFill(date.getMinutes())
		let seconds = DateTime.zeroFill(date.getSeconds())

		return format.replace('h', hours).replace('i', minutes).replace('s', seconds)
	}

	//个位数填充0
	static zeroFill(num: number) {
		return String(num < 10 ? '0' + num : num)
	}

	//获取日期，传入格式， yyyy年，yy两位数的年，mm月，dd天
	static getDate(d: DataParam, format = 'yyyy-mm-dd') {
		let date = DateTime.createDate(d)

		format = format.toLowerCase()
		let fullYear = date.getFullYear().toString()
		let year = fullYear.toString().substr(2, 2)
		let month = DateTime.zeroFill(date.getMonth() + 1)
		let day = DateTime.zeroFill(date.getDate())

		return format.replace('yyyy', fullYear).replace('yy', year).replace('mm', month).replace('dd', day)
	}

	//获取日期+时间
	static getDateTime(d: DataParam, format = 'yyyy-mm-dd h:i') {
		let date = DateTime.createDate(d)
		let formats = format.split(' ')
		let r = ff.dateTime.getDate(date, formats[0])
		if (formats[1]) {
			let time = ff.dateTime.getHm(date, formats[1])
			r = r + ' ' + time
		}
		return r
	}

	//获取周
	static getWeek(d: DataParam) {
		let date = DateTime.createDate(d)
		let week = date.getDay()
		let weekname = ['日', '一', '二', '三', '四', '五', '六']
		return weekname[week]
	}

	//返回日期数据 格式：[年,月,日,时,分,秒]
	static getDateTimeArray(d: DataParam) {
		let date = DateTime.createDate(d)

		let arr = ff.dateTime.getDateArray(date)
		let hours = date.getHours()
		let minutes = date.getMinutes()
		let seconds = date.getSeconds()

		arr.splice(3, 0, hours, minutes, seconds)
		return arr
	}

	//返回日期时间数据 格式：[年,月,日]
	static getDateArray(d: DataParam) {
		let date = DateTime.createDate(d)

		let fullYear = date.getFullYear()
		let month = date.getMonth() + 1
		let day = date.getDate()
		return [fullYear, month, day]
	}

	//返回年龄
	static getAge(d: DataParam) {
		let date = DateTime.createDate(d)

		let time = Date.now() - date.getTime()
		let age = new Date(time).getFullYear() - 1970
		return age
	}

	/*
	 * 获取一个天的时间戳，把时分秒都清零
	 * returns {Integer}
	 * */
	static getdaytimestamp(d: DataParam) {
		let date = DateTime.createDate(d)

		date.setHours(0)
		date.setMinutes(0)
		date.setSeconds(0)
		date.setMilliseconds(0)
		return date.getTime()
	}

	/*
	 * 获取今天的范围时间戳
	 * returns {Integer}
	 * */
	static getTodayTimetamps() {
		let date = new Date()

		date.setHours(0)
		date.setMinutes(0)
		date.setSeconds(0)
		date.setMilliseconds(0)

		let startTime = date.getTime()
		date.setHours(23)
		date.setMinutes(59)
		date.setSeconds(59)
		date.setMilliseconds(59)
		let endTime = date.getTime()

		return [startTime, endTime]
	}

	/*
	 * 获取本月时间的开始时间戳到结束时间戳
	 * @return [startTime,endTime]
	 * */
	static getmonthTimes() {
		let startDate = new Date()
		let endDate = new Date()
		startDate.setDate(0)
		let startTime = ff.dateTime.getdaytimestamp(+startDate)
		endDate.setMonth(endDate.getMonth() + 1)
		endDate.setDate(-1)
		let endTime = ff.dateTime.getdaytimestamp(+endDate)
		return [+startTime, +endTime]
	}

	/*
	 * 获取指定月的天数数组
	 * */
	static getMonthDays(month: number, year?: number) {
		let d = new Date(year || new Date().getFullYear(), month, 0)

		let ds = d.getDate()

		let days: { day: number; label: number | string }[] = [{day: 0, label: 'Day'}]
		for (let i = 1; i <= ds; i++) {
			days.push({day: i, label: i})
		}
		return days
	}

	static lang(): any {
		return {
			前: ' ago',
			后: ' after',
			刚刚: 'just now',
			不耗时: 'Not time consuming',
			秒: ' sec',
			分钟: ' minutes',
			小时: ' hr',
			天: ' days',
			今天: 'today'
		}
	}

	/**
	 * @param {Object} date 倒计日
	 * @returns {Number} 剩余天数，负数表示过期天数
	 */
	static pourMeterDay(d: DataParam, startTime?: number): number {
		const currentTime = startTime ? startTime : ff.dateTime.getdaytimestamp(new Date())
		const timeStamp = ff.dateTime.getdaytimestamp(d)

		// 判断传入时间戳是否早于当前时间戳
		const IS_EARLY = timeStamp < currentTime

		// 获取两个时间戳差值
		let diff = Math.abs(currentTime - timeStamp)

		let resStr: number
		if (diff < 86400000) {
			resStr = 0
		}
		// 多于23小时59分钟59秒
		else {
			let day = Math.floor(diff / 86400000)
			resStr = IS_EARLY ? -day : day
		}

		return resStr
	}

	/**
	 * @param {Object} date 日期对象
	 * @param {String} lang 语言 en|cn
	 * @param {Bollean} 可选 noVerb 没有动词 去除 前/后 动词
	 * @return {String} 相对时间字符串
	 */
	static getRelativeTime(d: DataParam, lang: 'cn' | 'en' = 'en', noVerb = false) {
		let date = DateTime.createDate(d)

		let langs: any = {}
		if (lang && lang === 'en') {
			langs = DateTime.lang()
		} else {
			Object.keys(DateTime.lang()).forEach(v => {
				langs[v] = v
			})
		}
		let currentYear = new Date().getFullYear()
		let year = date.getFullYear()
		const currentTime = Date.now()
		const timeStamp = date.getTime()

		// 判断传入时间戳是否早于当前时间戳
		const IS_EARLY = timeStamp <= currentTime

		// 获取两个时间戳差值
		let diff = currentTime - timeStamp
		// 如果IS_EARLY为false则差值取反
		if (!IS_EARLY) diff = -diff
		let resStr = ''
		let dirStr = IS_EARLY ? (noVerb ? '' : langs['前']) : noVerb ? '' : langs['后']
		if (diff < 1000) resStr = langs['刚刚']
		// 少于等于59秒
		else if (diff < 60000) resStr = Math.floor(diff / 1000) + langs['秒'] + dirStr
		// 多于59秒，少于等于59分钟59秒
		else if (diff >= 60000 && diff < 3600000) resStr = Math.floor(diff / 60000) + langs['分钟'] + dirStr
		// 多于59分钟59秒，少于等于23小时59分钟59秒
		else if (diff >= 3600000 && diff < 86400000) resStr = Math.floor(diff / 3600000) + langs['小时'] + dirStr
		// 多于23小时59分钟59秒，少于等于29天59分钟59秒
		else if (diff >= 86400000 && diff < 2623860000) resStr = Math.floor(diff / 86400000) + langs['天'] + dirStr
		// 多于29天59分钟59秒，少于364天23小时59分钟59秒，且传入的时间戳早于当前
		else if (diff >= 2623860000 && currentYear === year) resStr = ff.dateTime.getDateTime(date, 'mm-dd h:i')
		else resStr = ff.dateTime.getDateTime(date, 'yyyy-mm-dd h:i')

		return resStr
	}

	/**
	 * 传入秒，转成更直观的时间，如1分5秒，2时20分18秒，1天4时1分43秒等
	 * @param second 秒
	 * @param lang 语言 en|cn
	 * @returns {String} 相对时间字符串
	 */
	static getIntuitiveTime(second: number, lang: 'cn' | 'en' = 'en') {
		let langs: any = {}
		if (lang && lang === 'en') {
			langs = DateTime.lang()
		} else {
			Object.keys(DateTime.lang()).forEach(v => {
				langs[v] = v
			})
		}

		let resStr = ''

		if (second < 1) resStr = langs['不耗时']
		// 少于等于59秒
		else if (second < 60) resStr = second + langs['秒']
		// 多于59秒，少于等于59分钟59秒
		else if (second >= 60 && second < 3600) {
			let i = Math.floor(second / 60) + langs['分钟']

			resStr = i + ' ' + (second % 60) + langs['秒']
		}
		// 多于59分钟59秒，少于等于23小时59分钟59秒
		else if (second >= 3600 && second < 86400) {
			let h = Math.floor(second / 3600) + langs['小时']
			let temp = second % 3600
			let i = Math.floor(temp / 60) + langs['分钟']
			resStr = h + ' ' + i
		}
		// 多于23小时59分钟59秒，少于等于29天59分钟59秒
		else if (second >= 86400) {
			let d = Math.floor(second / 86400) + langs['天']
			let temp = second % 86400
			let h = Math.floor(temp / 3600) + langs['小时']

			resStr = d + ' ' + h
		}

		return resStr
	}

	/*
	 * 生成英文格式的，生日 如：February 15, 2021
	 * @params {month,day,year} 生日数据
	 * @param shorthand 是生启用简写
	 * @returns string 生日字面字符串
	 * */
	static getEnDate(birthData: { month?: f2.Month | 0; day?: number; year?: number }, shorthand = false) {
		let str: string = ''
		if (birthData.month) {
			let d = monthEn[birthData.month]
			if (shorthand) {
				str = d.abbr
			} else {
				str = d.name
			}

			if (birthData.day) str += ' ' + birthData.day
			if (birthData.year) str += (birthData.day ? ', ' : ' ') + birthData.year
		} else if (birthData.year) {
			str = `${birthData.year}`
		}

		return str
	}

	//获取指定天数后的时间戳，为0时，获取当天0时0分0秒的时间戳
	//@param {number} day 天数
	static getAfterDay(day: number = 0) {
		let date = new Date()
		date.setDate(date.getDate() + day)
		date.setHours(0)
		date.setMinutes(0)
		date.setSeconds(0)
		date.setMilliseconds(0)
		return Math.floor(date.getTime() / 1000)
	}

	//获取指定时区的时间
	static getTimeInTimeZone(timeZone: number) {
		let t = new Date()
		let offset = t.getTimezoneOffset() * 60 * 1000,
				wTime = t.getTime() + offset,
				newTime = wTime + timeZone * 3600000

		t.setTime(newTime)
		let data: (string | number)[] = DateTime.getDateTimeArray(t)
		data[4] = data[4].toString().padStart(2, '0')
		data[3] = data[3].toString().padStart(2, '0')
		return data
	}
}

export default DateTime
