//在渲染器进程 (网页) 中。
/*
 * mqtt 渲染端处理
 * */

/*
 * 键：表示已经订阅的主题
 * 值：是一个数组，执行函数
 * */

const topics: {[topic: string]: Function[]} = {}

//当前渲染层id
const webContentId: number = window._pre_.getWebContentId()

interface RendererClient extends ff.gc.Client {
  sortingMessages(topic: string, ...msg: any): void
}

const gc: RendererClient = {
  /*
   * 分拣消息,分配消息通知到指定的位置
   * @param {String} topic 主题
   * @param {mixin} message 消息
   * */
  async sortingMessages(topic: string, ...message: any) {
    ff.util.devlog('%c收到fgc: ' + topic, 'background:#3F6FFB;color:#fff', ...message)

    //获取订阅主题的数量
    let funs: Function[] = []

    //直接配置
    if (topics[topic] && topics[topic].length) {
      funs = funs.concat(topics[topic])
    }

    if (!funs.length) {
      ff.util.devlog('%c没有订阅该主题: ' + topic, 'background:#E86C5D;color:#fff')
      return
    }

    if (funs.length) {
      funs.forEach(func => {
        typeof func === 'function' && func(...message)
      })
    }
  },

  /*
   * 发送一个本地的mqtt,只会通知本机的订阅者
   * @param {String} topic 主题
   * @param {mixni} message 内容
   * */
  send(topic, ...message) {
    window._pre_.gc.send(topic, ...message)
  },

  /*
   * 发送一个ipc,只发送到主进程
   *
   * */
  sendMain(topic, ...message) {
    window._pre_.gc.sendMain(topic, ...message)
  },

  //在渲染进程不起作用
  sendToWebContents(webConents, topic, ...message) {
    window._pre_.gc.send(topic, ...message)
  },

  /*
   * 只发送给当前页面或当前进程（只有当前页面能收到通知）
   * 该方法不用跨进程，所以性能也是最好的
   * */
  sendCurrent(topic, ...message) {
    gc.sortingMessages(topic, ...message)
  },

  //renderer中添加事件
  //@param {Boolean} 不用向主进程订阅
  on(topicName, cb, noSendSub) {
    if (!noSendSub) {
      window._pre_.gc.subscribes([topicName], webContentId)
    }

    let topic = topics[topicName]
    if (topic) {
      //对比两个函数是否一样
      let isRepetition = topic.some(v => v === cb)

      //把执行函数加到对应的主题上
      if (!isRepetition) {
        topic.push(cb)
      }
    } else {
      //把执行函数加到对应的主题上
      topics[topicName] = [cb]
    }
  },

  //订阅
  subscribes(subs) {
    let tops = Object.keys(subs) //主题数组
    window._pre_.gc.subscribes(tops, webContentId)
    for (let i in subs) {
      let fun = subs[<ff.gc.IpcTopic>i]
      if (fun) gc.on(i, fun, true)
    }
  },

  //取消订阅
  unsubscribes(subs: Partial<ff.gc.Subscribes>) {
    //存放需要取消订阅的主题
    //不是所有的主题都要取消订阅，有些主题可能存在多个function,
    //只有剩下一个function的时候，才是真正取消订阅
    let unSubList: string[] = []

    for (let i in subs) {
      if (topics[i]) {
        let topic = topics[i]

        if (topic.length > 1) {
          topic.some((v, index) => {
            if (v === subs[<ff.gc.IpcTopic>i]) {
              topic.splice(index, 1)
              return true
            }
          })
        } else if (topic.length === 1 && topic[0] === subs[<ff.gc.IpcTopic>i]) {
          unSubList.push(i)
          delete topics[i]
        }
      } else {
        unSubList.push(i)
      }
    }

    if (unSubList.length) window._pre_.gc.unsubscribes(unSubList, webContentId)
  }
}

//自动向父组件注册该渲染层，使该渲染层能收到通知
window._pre_.gc.onmessage((topic: string, ...message: any) => {
  gc.sortingMessages(topic, ...message)
})

window.addEventListener('unload', () => {
  window._pre_.gc.removeClient(webContentId)
})

export default gc
